#include "arduinOSC.h"


void arduinOSC::processInterrupt();
void arduinOSC::initOSC(uint8_t wf);	
void arduinOSC::setOSCFrequency(float frequency);

boolean arduinOSC::pendingRequest;
boolean arduinOSC::interruptDisabled;
float arduinOSC::pendingFrequency;
float arduinOSC::normalisationFactor;
float arduinOSC::frequency;
uint8_t arduinOSC::dutyCycleValue;
uint8_t arduinOSC::waveform;
uint8_t arduinOSC::indexWidth;
uint8_t arduinOSC::i;
uint16_t arduinOSC::timerICR;
uint16_t arduinOSC::dutyCycleRegister;

	
void arduinOSC::initOSC(uint8_t wf){
	pendingRequest = false;
	interruptDisabled = true;
	pendingFrequency = 0;
	normalisationFactor = 1.0;
	frequency = 0;
	timerICR = 0;
	indexWidth = 1;
	waveform = wf;
	setDutyCycle(50);
	i=0;
	DDRD=0x00;
	PORTD=0xFF;
	DDRB=0xFF;
	TCCR1A =  (1<<COM1A1)|(0<<COM1A0)|(0<<COM1B1)|(0<<COM1B0)|(0<<0)|(0<<0)|(1<<WGM11) |(0<<WGM10);
	if(waveform==WAVEFORM_SQUARE){
		TCCR1B =  (0<<ICNC1) |(0<<ICES1) |(1<<WGM13) |(1<<WGM12) |(0<<CS12)    |(1<<CS11)  |(1<<CS10); 
	}
	else{
		TCCR1B =  (0<<ICNC1) |(0<<ICES1) |(1<<WGM13) |(1<<WGM12) |(0<<CS12)    |(0<<CS11)  |(1<<CS10); 
	}
	TIMSK1=0x20;
	TCNT1 = 0;
	
}

void arduinOSC::applyOSCFrequency(){
	switch (waveform){

		case WAVEFORM_SINE:
		case WAVEFORM_USER:
			if(pendingFrequency>4000) indexWidth = 64;
			else if(pendingFrequency>2000) indexWidth = 32;
			else if(pendingFrequency>1000) indexWidth = 16;
			else if(pendingFrequency>500) indexWidth = 8;
			else if(pendingFrequency>250) indexWidth = 4;
			else if(pendingFrequency>125) indexWidth = 2;
			else indexWidth=1;
			timerICR = (int)(62500.0/(float)(pendingFrequency/(float)indexWidth));
			ICR1 = (int)timerICR;
			normalisationFactor = timerICR/255.0;
			if(interruptDisabled) OCR1A=(int)(normalisationFactor*0x1);
			break;
		case WAVEFORM_SQUARE:
		/*	if(pendingFrequency>5000){
				TCCR1B =  (0<<ICNC1) |(0<<ICES1) |(1<<WGM13) |(1<<WGM12) |(1<<CS12)    |(0<<CS11)  |(0<<CS10); 
				timerICR = (6250.0/(float)pendingFrequency*10.0);
			}
			else{
				TCCR1B =  (0<<ICNC1) |(0<<ICES1) |(1<<WGM13) |(1<<WGM12) |(0<<CS12)    |(1<<CS11)  |(1<<CS10);
				timerICR = (2500.0/(float)pendingFrequency*100.0);			
			}*/
			timerICR = (2500.0/(float)pendingFrequency*100.0);
			ICR1 = (int)timerICR;
			dutyCycleRegister = (dutyCycleValue/100.0)*ICR1;
			OCR1A=dutyCycleRegister;
			
			break;
	}
}

void arduinOSC::setOSCFrequency(float frequency){
  pendingRequest = true;
  pendingFrequency = frequency;
  if(interruptDisabled) applyOSCFrequency();
}
void arduinOSC::processInterrupt(){
  if(interruptDisabled) interruptDisabled = false;

  
switch (waveform){

	case WAVEFORM_SINE:
		if(pendingRequest&&i==2){
		  pendingRequest = false;
		  applyOSCFrequency();
		}
		OCR1A=sinewave_coeff[i]*normalisationFactor;
		i+=indexWidth;
		break;
	case WAVEFORM_SQUARE:
		if(pendingRequest){
		  pendingRequest = false;
		  applyOSCFrequency();
		}
		break;
	case WAVEFORM_USER:
		if(pendingRequest&&i==2){
		  pendingRequest = false;
		  applyOSCFrequency();
		}
		OCR1A=user_waveform[i]*normalisationFactor;
		i+=indexWidth;
		break;
  }

}

void arduinOSC::setDutyCycle(uint8_t dCycle){
	if(waveform==WAVEFORM_SQUARE){
	dutyCycleValue = dCycle;
		if(dutyCycleValue>100)dutyCycleValue = 100;
		dutyCycleRegister = (dutyCycleValue/100.0)*ICR1;
		OCR1A = dutyCycleRegister;
	}
}

ISR(TIMER1_CAPT_vect){
arduinOSC::processInterrupt();
}